package com.savagehomeautomation.devoxx;

import android.app.Activity;
import android.os.Bundle;
import android.os.StrictMode;
import android.view.View;
import android.widget.Toast;
import com.google.gson.Gson;
import org.eclipse.paho.client.mqttv3.*;

import java.net.UnknownHostException;

/**
 * @author Robert Savage
 * @see http://www.pi4j.com
 * @see http://www.savagehomeautomation.com
 * @see https://twitter.com/savageautomate
 *
 */
public class Pi4J_Demo extends Activity {

    protected MqttAsyncClient mqtt;

    /**
     * Called when the activity is first created.
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {

        // allow network access on main thread
        // (NOTE: YOU SHOULD NOT DO THIS IN A PRODUCTION APP, THIS IS JUST FOR DEMO PURPOSES!!)
        StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();
        StrictMode.setThreadPolicy(policy);

        // create main activity
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        // (NOTE: YOU SHOULD NOT USE THE PAHO JAVA MQTT CLIENT DIRECTLY LIKE THIS IN ANDROID PROJECTS!
        //        I AM USING THIS JUST TO KEEP THIS DEMO CODE SIMPLE AND LIGHTWEIGHT.
        //        YOU SHOULD INSTEAD USE THE PAHO ANDROID MQTT SERVICE.
        //        http://www.eclipse.org/paho/clients/android/
        // )

        // create MQTT client instance
        try {
            mqtt = MqttHelper.createAsync();
        } catch (MqttException e) {
            e.printStackTrace();
        } catch (UnknownHostException e) {
            e.printStackTrace();

            // display error message
            Toast.makeText(getApplicationContext(), e.getMessage(), Toast.LENGTH_LONG).show();
        }
    }

    /**
     * Called when the activity is started or re-started
     */
    @Override
    public void onStart(){
        super.onStart();

        System.out.println(">>> STARTED <<<");

        // get an instance to each GUI alert view container
        final View drivewayAlert = findViewById(R.id.drivewayAlert);
        final View floodAlert = findViewById(R.id.floodAlert);
        final View hvacAlert = findViewById(R.id.hvacAlert);
        final View mailboxAlert = findViewById(R.id.mailboxAlert);

        // when starting up, all alert view containers should be hidden
        drivewayAlert.setVisibility(View.INVISIBLE);
        floodAlert.setVisibility(View.INVISIBLE);
        hvacAlert.setVisibility(View.INVISIBLE);
        mailboxAlert.setVisibility(View.INVISIBLE);

        try {
            // connect to MQTT broker (server)
            MqttHelper.connect(mqtt, new IMqttActionListener() {
                @Override
                public void onSuccess(IMqttToken iMqttToken) {
                    try {
                        // subscribe to the MQTT topic
                        mqtt.subscribe(MqttHelper.TOPIC, 2);
                    } catch (final MqttException e) {
                        e.printStackTrace();

                        // display error message
                        Pi4J_Demo.this.runOnUiThread(new Runnable() {
                            public void run() {
                                Toast.makeText(getApplicationContext(), e.getMessage(), Toast.LENGTH_LONG).show();
                            }
                        });
                    }
                }

                @Override
                public void onFailure(IMqttToken iMqttToken, Throwable throwable) {
                }
            });

            // handle MQTT callbacks
            mqtt.setCallback(new MqttCallback() {

                @Override
                public void connectionLost(Throwable cause) {
                    System.out.println("[MQTT] --- (CONNECTION LOST) " + cause.getMessage());
                }

                @Override
                public void messageArrived(String topic, MqttMessage message) throws Exception {
                    System.out.println("[MQTT] <<< (RECEIVED) " + message);
                    try {
                        // attempt to deserialize message payload from MQTT to JSON string and into sensor states object
                        final SensorStates sensorStates = new Gson().fromJson(new String(message.getPayload(), "UTF-8"), SensorStates.class);

                        // run GUI update on UI thread
                        Pi4J_Demo.this.runOnUiThread(new Runnable() {
                            public void run() {
                                // set alert notification visibility (on message received)
                                drivewayAlert.setVisibility(sensorStates.drivewayTripped ? View.VISIBLE : View.INVISIBLE);
                                floodAlert.setVisibility(sensorStates.floodTripped ? View.VISIBLE : View.INVISIBLE);
                                hvacAlert.setVisibility(sensorStates.hvacTripped ? View.VISIBLE : View.INVISIBLE);
                                mailboxAlert.setVisibility(sensorStates.mailboxTripped ? View.VISIBLE : View.INVISIBLE);
                            }
                        });
                    } catch (final Exception ex) {
                        ex.printStackTrace();

                        // display error message
                        Pi4J_Demo.this.runOnUiThread(new Runnable() {
                            public void run() {
                                Toast.makeText(getApplicationContext(), ex.getMessage(), Toast.LENGTH_LONG).show();
                            }
                        });
                    }
                }

                @Override
                public void deliveryComplete(IMqttDeliveryToken token) {
                    // do nothing here; we are not sending messages in this app
                }

            });
        }
        catch(MqttException me) {
            me.printStackTrace();

            // display error message
            Toast.makeText(this, me.getMessage(), Toast.LENGTH_LONG).show();
        }
    }

    /**
     * Called when the activity is stopped
     */

    @Override
    public void onStop(){
        super.onStop();
        try {
            // disconnect from MQTT broker
            MqttHelper.disconnect(mqtt);
        } catch (MqttException e) {
            e.printStackTrace();

            // display error message
            Toast.makeText(this, e.getMessage(), Toast.LENGTH_LONG).show();
        }
    }
}
